USE Market
GO

if exists (select * from sysobjects where id = object_id('dbo.AddOrder'))
  drop procedure dbo.AddOrder
GO


if exists (select * from sysobjects where id = object_id('dbo.Customers'))
  drop table dbo.Customers
GO


if exists (select * from sysobjects where id = object_id('dbo.Orders') and sysstat & 0xf = 3)
  drop table dbo.Orders
GO


if exists (select * from sysobjects where id = object_id('dbo.Products') and sysstat & 0xf = 3)
  drop table dbo.Products
GO


CREATE TABLE dbo.Customers (
	Customer varchar (30) NOT NULL ,
	CreditLimit money NOT NULL CONSTRAINT CreditLimitDF DEFAULT (0),
	AccountBalance money NOT NULL CONSTRAINT AcountBalanceDF DEFAULT (0),
	CONSTRAINT CustomersPK PRIMARY KEY  CLUSTERED (Customer)
)
GO

GRANT  REFERENCES ,  SELECT ,  INSERT ,  DELETE ,  UPDATE  ON Customers  TO public
GO


CREATE TABLE dbo.Orders (
	OrderNumber int IDENTITY (1000, 1) NOT NULL ,
	Customer varchar (30) NOT NULL ,
	Product varchar (30) NOT NULL ,
	Quantity int NOT NULL ,
	Price money NULL ,
	CONSTRAINT OrdersPK PRIMARY KEY  NONCLUSTERED (OrderNumber)
)
GO

GRANT  REFERENCES ,  SELECT ,  INSERT ,  DELETE ,  UPDATE  ON Orders  TO public
GO

CREATE TABLE dbo.Products (
	Product varchar (20) NOT NULL ,
	Price money NOT NULL ,
	Quantity int NOT NULL CONSTRAINT QuantityDF DEFAULT (0),
	CONSTRAINT ProductsPK PRIMARY KEY  CLUSTERED (Product)
)
GO

GRANT  REFERENCES ,  SELECT ,  INSERT ,  DELETE ,  UPDATE  ON Products  TO public
GO

CREATE PROCEDURE AddOrder 
  @Customer varchar(30),
  @Product varchar(30),
  @Quantity int,
  @Price money,
  @OrderNumber int OUTPUT
AS

SELECT @OrderNumber = 0

INSERT 
  INTO Orders(Customer,
              Product,
              Quantity,
              Price)
  VALUES(@Customer,
         @Product,
         @Quantity,
         @Price)

SELECT @OrderNumber = @@IDENTITY
GO

GRANT  EXECUTE  ON dbo.AddOrder  TO public
GO


INSERT INTO Products VALUES('Apples', 0.69, 10000)
INSERT INTO Products VALUES('Boomerangs', 9.95, 5000)
INSERT INTO Products VALUES('Canaries', 29.95, 1000)
INSERT INTO Products VALUES('Doohickies', 75.00, 1000)
INSERT INTO Products VALUES('Ear mittens', 189.95, 100)
INSERT INTO Products VALUES('Fudruckers', 399.99, 50)
INSERT INTO Products VALUES('Gates dolls', 650, 10)


INSERT INTO Customers VALUES('Fred', 1000, 0)
INSERT INTO Customers VALUES('Mary', 1000, 0)
INSERT INTO Customers VALUES('Louis', 1000, 0)
INSERT INTO Customers VALUES('Wanda', 1000, 0)

DECLARE @LocalHost varchar(255)
SELECT @LocalHost = @@SERVERNAME
INSERT INTO Customers VALUES(@LocalHost, 1000, 0)